#include "testlib.h"
#include <map>
#include <vector>

using namespace std;

struct Edge {
    int a, b, c;
};

int N, M;
vector<Edge> edges;

const double EPS = 1e-6;

pair<bool, double> readAns(InStream& stream) {

    auto value = stream.readWord();
    stream.ensuref(value == "YES" || value == "NO" , "Expected YES or NO");

    bool solution = (value == "YES");

    if (!solution) {
        return {false, -1};
    }

    double totalsum = 0;

    vector<double> values;
    for (int i = 1; i <= N; i++)
    {
        double value = stream.readDouble();
        values.push_back(value);
        totalsum += abs(value);
    }

    for (auto e : edges) {
        double res = values[e.a - 1] + values[e.b - 1];
        if (abs(e.c - res) > EPS) {
            stream.quitf(
                _wa,
                "Sum of endpoints for edge (%d; %d) differs from the expected value %d.",
                e.a,
                e.b,
                e.c
            );
        }
    }

    return {true, totalsum};
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);
    N = inf.readInt();
    M = inf.readInt();
    for (int i = 0; i < M; i++) {
        int a = inf.readInt();
        int b = inf.readInt();
        int c = inf.readInt();
        edges.push_back({
            a, b, c
        });
    }

    bool jans, pans;
    double jansv, pansv;
    tie(jans, jansv) = readAns(ans);
    tie(pans, pansv) = readAns(ouf);
    if (jans && !pans)
    {
        quitf(_wa, "jury has the better answer: jans = YES, pans = NO\n");
    }
    else if (jans == pans)
    {
        if (
                jans == false // answer is NO
                || abs(jansv - pansv) <= EPS // answer is YES and matches
           )
        {
            quitf(_ok, "answer = %s\n", jans ? "YES" : "NO");
        }
        else if (jansv < pansv)
        {
            quitf(_wa, "participant answer is larger than the answer of jury");
        }
        else if (abs(jansv - pansv) <= 0.5)
        {
            quitf(_wa, "participant answer is too inprecise");
        }
        else
        {
            quitf(_fail,
                  "participant has an solution with better answer for the sum of absolute value of all assigned numbers."
                  " jury %f; participant %f;", jansv, pansv);
        }
    }
    else // (jans < pans)
    {
        quitf(_fail,
              ":( participant has the better answer: jans = %s, pans = %s\n",
              jans ? "YES" : "NO",
              pans ? "YES" : "NO");
    }
}
